<table border="0" cellspacing="0" cellpadding="0" class="host_include">
  <tr>
    <td rowspan="4" valign="middle"><img
      src="/images/rhel5-small.png" width="85" height="63"
      alt="RedHat Enterprise Linus 5" /></td>
      <th>Location: &nbsp; </th><td>RedBus Soverign House - Xcelsys Ltd.                 </td></tr>
  <tr><th>O/S:             </th><td>RedHat Enterprise Linux 5 ES (i386)                  </td></tr>
  <tr><th>CPU:             </th><td>1x Dual-Core AMD Opteron(tm) Processor 2216 (virtual)</td></tr>
  <tr><th> &nbsp;          </th><td> &nbsp;                                              </td></tr>
</table>
