<table border="0" cellspacing="0" cellpadding="0" class="host_include">
  <tr>
    <td rowspan="4" valign="middle"><img
      src="/images/NetBSD-small.png" width="85" height="63"
      alt="NetBSD" /></td>
      <th>Location: &nbsp; </th><td>Telecity Bromma, Stockholm                           </td></tr>
  <tr><th>O/S:             </th><td>NetBSD 3.1 (i386)                                    </td></tr>
  <tr><th>CPU:             </th><td>1x Genuine Intel(R) CPU           T1300  @ 1.66GHz   </td></tr>
  <tr><th> &nbsp;          </th><td> &nbsp;                                              </td></tr>
</table>
