<table border="0" cellspacing="0" cellpadding="0" class="host_include">
  <tr>
    <td rowspan="4" valign="middle"><img
      src="/images/Cisco-small.png" width="85" height="63"
      alt="Cisco Catalyst" /></td>
      <th>Location: &nbsp; </th><td>Technology House: 3rd Floor                          </td></tr>
  <tr><th>O/S:             </th><td>Cisco IOS Software Release 12.2(18)EW                </td></tr>
  <tr><th>Device:          </th><td>Cisco Catalyst 4510R                                 </td></tr>
  <tr><th> &nbsp;          </th><td> &nbsp;                                              </td></tr>
</table>
